//-------------------------------------------------------------
// Author: William Hammontree
// Date of most recent update: 12/10/15
// 
// This code was created as a part of the USF MAKE course. 
//------------------------------------------------------------

//Precomp Statements

#include <Servo.h> 
#include "RunningAverage.h"
#include <SPI.h> //This library is for SPI commuication to the digital potentiometers
#include <Wire.h> //These two libraries are needed for the LCD screen
#include <LiquidCrystal_I2C.h>

//Initialize Running Average
RunningAverage myRA(10);
int samples = 0;

//Initialize Servo
Servo myservo;

//Initialize Slave for SPI Digital Pot
const int slaveSelect=10;

//Initialize LCD Screen
LiquidCrystal_I2C myDisplay(0x27,16,2);

//Declaring Global Variables
int percent;
int angle;
int micpercent;
long micReading;
unsigned long samplewindow=500;
unsigned long windowstart=millis();
unsigned int peakpeak=0;
unsigned int signalmax=0;
unsigned int signalmin=1024;
long oldPot=0;
double potOut=0;



// -----------------------------------------------------------------------------------------
void setup() {
  
  // Set Up LCD Display
  myDisplay.init();           
  myDisplay.backlight();
  
 // Set Up Servo
  myservo.attach(8);

 // Set Up Slave Pin
 pinMode(slaveSelect, OUTPUT);
 SPI.begin();

 // Set Up Running Average
  myRA.clear(); // explicitly start clean

  // Start Serial Communication
  Serial.begin(9600);
}
//-----------------------------------------------------------------------------------------
void loop() {

 windowstart=millis(); // This portion resets the window value and the max and min values at the start of each loop
 signalmax=0;
 signalmin=1024;

  // GET SAMPLE: This finds a maximum and a minimum value for a set window 
  // then subtracts them to find the peak to peak value ----------------------------------------
while (millis()-windowstart<samplewindow) {
  micReading= analogRead(0);
  if (micReading>signalmax) {
    signalmax=micReading;
  };
  if(micReading<signalmin) {
    signalmin=micReading;
  };
};

peakpeak=signalmax-signalmin;
Serial.print(signalmax);
Serial.print("--");
Serial.print(signalmin);
Serial.print("--");
Serial.print(peakpeak);

// MANIPULATE SAMPLE: this converts the sample from a raw reading to a usable number
// which will be output to the servo and the digital pot

micpercent=map(peakpeak,75,350,0,100);  
Serial.print(micpercent);
int potIn=analogRead(1);
potOut=map(potIn,0,1028,50,200);
double ratio=potOut/100;
percent=(micpercent)*ratio;
Serial.print("-");
Serial.print(percent);
if(percent>100){
  percent=100;
}

angle=map(percent,0,100,180,20);
Serial.print("--");
Serial.print(angle);
Serial.print("--");

// RUNNING AVERAGE: the running average slows down the response of the system and
// makes small variations in the mic signal less impactful
myRA.addValue(angle);
samples++;
Serial.print(samples);
Serial.print("\t Running Average: ");
int runningAvg=myRA.getAverage();
if (runningAvg>180){
  runningAvg=180;
}
if (runningAvg<0){
  runningAvg=0;
}
Serial.println(runningAvg);
  
potOut=map(runningAvg,180,45,155,0);

// ACTIONS: this portion of the code exectutes the outputs based on the manipulated
// samples

//Print the ratio to the LCD.
 myDisplay.clear();
 myDisplay.setCursor(0,0);
  myDisplay.print("Music:NoiseRatio");
 myDisplay.setCursor(4,1);
  myDisplay.print(ratio);
  myDisplay.print(" : 1");

// Change position of the needle. Only changes if the difference is greater than 5%
if(abs((potOut-oldPot)/oldPot)>.05){
// Change the resistance provided by the digital pots. Only changes if the differnce is greater than 5%
digitalWrite(slaveSelect, LOW);
SPI.transfer(potOut);
digitalWrite(slaveSelect,HIGH);

myservo.write(runningAvg);

oldPot=potOut;
}

delay(1000);
Serial.println(" ");

}

